﻿Namespace ExpenseReport.Data
	Friend Class ExpenseReportManager
		Private _Context As ExpenseReportDatabaseEntities
		Private ReadOnly Property Context() As ExpenseReportDatabaseEntities
			Get
				If _Context Is Nothing Then
					_Context = New ExpenseReportDatabaseEntities()
				End If

				Return _Context
			End Get
		End Property

        Friend Function CreateExpenseReport(ByVal amount As Decimal,
                                            ByVal description As String) As Integer
            Dim expenseReport As New ExpenseReport() With {.Amount = amount,
                                                           .Description = description,
                                                           .Status = ExpenseReportStatus.Created}

            Context.AddToExpenseReports(expenseReport)
            Context.SaveChanges()

            Return expenseReport.Id
        End Function

		Friend Sub ApproveExpenseReport(ByVal id As Integer)
			UpdateExpenseReportStatus(id, ExpenseReportStatus.Approved)
		End Sub

		Friend Sub DenyExpenseReport(ByVal id As Integer)
			UpdateExpenseReportStatus(id, ExpenseReportStatus.Denied)
		End Sub

		Friend Function GetPendingExpenseReports() As ExpenseReport()
			Return (
			    From er In Context.ExpenseReports
			    Where er.Status = ExpenseReportStatus.Created
			    Select er).ToArray()
		End Function

		Private Sub UpdateExpenseReportStatus(ByVal id As Integer, ByVal status As String)
			Dim expenseReport As ExpenseReport = GetExpenseReport(id)

			expenseReport.Status = status
			Context.SaveChanges()
		End Sub

		Private Function GetExpenseReport(ByVal id As Integer) As ExpenseReport
			Return (
			    From er In Context.ExpenseReports
			    Where er.Id = id
			    Select er).FirstOrDefault()
		End Function
	End Class
End Namespace